/***************************************************************************
 *   This file is part of Webbino                                          *
 *                                                                         *
 *   Copyright (C) 2012-2016 by SukkoPera                                  *
 *                                                                         *
 *   Webbino is free software: you can redistribute it and/or modify       *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Webbino is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with Webbino. If not, see <http://www.gnu.org/licenses/>.       *
 ***************************************************************************/

#ifndef WEBBINOCOMMON_H_
#define WEBBINOCOMMON_H_

/* Network device selection: please enable *only one* of the following,
 * corresponding to the network device you will be using
 */
#define WEBBINO_USE_WIZ5100
//~ #define WEBBINO_USE_WIZ5500
//~ #define WEBBINO_USE_ENC28J60
//~ #define WEBBINO_USE_ESP8266
//~ #define WEBBINO_USE_ESP8266_STANDALONE
//~ #define WEBBINO_USE_WIFI
//~ #define WEBBINO_USE_WIFI101
//~ #define WEBBINO_USE_DIGIFI

/* Define to enable serving webpages from SD. This will use Arduino's SD
 * library, which only allows DOS-style (i.e. 8+3 characters) file names. This
 * means that you will have to name your pages with a .htm extension, instead of
 * .html. If you don't like this, install the SdFat library and see below.
 */
//~ #define WEBBINO_ENABLE_SD

/* Define to enable serving webpages from SD. This will use the SDFat
 * library (https://github.com/greiman/SdFat). This library allows access to
 * files with long names (LFNs), if properly configured (see SdFatConfig.h in
 * the library sources).
 */
//~ #define WEBBINO_ENABLE_SDFAT

/* Define to enable support for tag substitutions, i.e.: replace #TAGS#
 * in served pages
 */
#define ENABLE_TAGS

/* Character that delimits tags
 */
#define TAG_CHAR '#'

/* Maximum length of a tag name
 */
#define MAX_TAG_LEN 24

/* Maximum length of a string generated by tag substitution
 */
#define MAX_SUBSTITUTION_LEN 32

/* Maximum length of a get parameter name and value
 */
#define BUF_LEN 32

/* Maximum length of an URL to precess
 */
#define MAX_URL_LEN 128

/* TCP port the server will listen on
 *
 * NOTE: Port 80 can not be used with DigiFi
 * NOTE: Currently changing this will have no effect with most cards, FIXME
 */
#define SERVER_PORT 8000

/* Name of the index page, i.e. the page requests for / get redirected to.
 */
#define REDIRECT_ROOT_PAGE "index.html"

/* Define this to store strings in flash memory. This saves RAM on smaller MCUs,
 * recommended on AVRs, works fine on ESP8266 standalone, probably not supported
 * on other targets.
 */
#if defined (ARDUINO_ARCH_AVR) || defined (ESP8266)
	#define ENABLE_FLASH_STRINGS
#endif

/* DEFINE this to DISABLE debug messages
 */
#define WEBBINO_NDEBUG

/* Enable verbose HTTP request parser
 */
//~ #define VERBOSE_REQUEST_PARSER

// Don't touch :)
#define WEBBINO_VERSION "0.9.0"

#endif
